/*
  ******************************************************************************
  * @file    apt32f103_usart.c
  * @author  
  * @version V1.00
  * @date    2023/02/01
  ******************************************************************************
  *THIS SOFTWARE WHICH IS FOR ILLUSTRATIVE PURPOSES ONLY WHICH PROVIDES 
  *CUSTOMER WITH CODING INFORMATION REGARDING THEIR PRODUCTS.
  *APT CHIP SHALL NOT BE HELD RESPONSIBILITY ADN LIABILITY FOR ANY DIRECT, 
  *INDIRECT DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF 
  *SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION 
  *CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.AND APT CHIP RESERVES 
  *THE RIGHT TO MAKE CHANGES IN THE SOFTWARE WITHOUT NOTIFICATION
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/ 
#include "apt32f103_usart.h"
/* defines -------------------------------------------------------------------*/
#define LOOPBUFSIZE ((U16_T) 0x20)
/* -------- variables ---------------------------------------------------------*/
volatile U8_T RxDataBuf[LOOPBUFSIZE+4];

volatile U8_T USART_RxDataFlag;
volatile U8_T USART_TxDataFlag;
volatile U8_T byLin0EerorFlg,byLin0EndMessFlg,byLin1EndMessFlg,byLin0SendHold,byLin1SendHold;
volatile U8_T byRecLin0Data[8];
volatile U8_T byRecLin1Data[8];
/* externs--------------------------------------------------------------------*/


/*************************************************************/
//USART RESET,CLEAR ALL REGISTER
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_DeInit(void)
{
	USART0->IDR   	= USART_IDR_RST;
	USART0->CEDR   	= USART_CEDR_RST;
	USART0->SRR 	= USART_SRR_RST;
	USART0->CR 		= USART_CR_RST;
	USART0->MR   	= USART_MR_RST;
	USART0->IMSCR   = USART_IMSCR_RST;
	USART0->RISR  	= USART_RISR_RST;
	USART0->MISR   	= USART_MISR_RST;
	USART0->ICR   	= USART_ICR_RST;
	USART0->SR   	= USART_SR_RST;
	USART0->RHR   	= USART_RHR_RST;
	USART0->THR   	= USART_THR_RST;
	USART0->BRGR   	= USART_BRGR_RST;
	USART0->RTOR   	= USART_RTOR_RST;
	USART0->TTGR   	= USART_TTGR_RST;
	USART_Software_Reset();
}
/*************************************************************/
//USART DMA Control
//EntryParameter:USART_Tx_Rx_CMD,UART_TX_FL_X,UART_RX_FL_X
//USART_Tx_Rx_CMD:BIT0(RX dma cmd);BIT1(TX dam cmd)
//USART_TX_FL_X:USART_TX_FL_FIFO_NF,USART_TX_FL_Less_FIFO_1_2
//USART_RX_FL_X:USART_RX_FL_FIFO_NE,USART_RX_FL_FIFO_FLSEL
//ReturnValue:NONE
/*************************************************************/
void USART_DMA_Control(U8_T USART_Tx_Rx_CMD , USART_TX_DMA_SEL_TypeDef USART_TX_FL_X , USART_RX_DMA_SEL_TypeDef USART_RX_FL_X)
{
	USART0->DMACR=(USART0->DMACR&0xfffffff0)|USART_Tx_Rx_CMD|USART_TX_FL_X|USART_RX_FL_X;
}
/*************************************************************/
//USART Interrupt enable
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_Int_Enable(void)
{
    USART0->ICR=0x7FF;									//clear USART INT status
	csi_vic_enable_irq(USART0_IRQ_NUM);							//INT Vector Enable USART Interrupt
}
/*************************************************************/
//USART Interrupt enable
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_Int_Disable(void)
{
	csi_vic_disable_irq(USART0_IRQ_NUM);							//INT Vector Enable USART Interrupt
}
/*************************************************************/
//USART IO Init
//EntryParameter:USART_IO_G
//USART_IO_G:0 1
//ReturnValue:NONE
/*************************************************************/
void USART_IO_Init(USART_NUM_TypeDef USART_IO_G)
{
    if (USART_IO_G==USART_TxPB0_10_RxPB0_11)
    {
		GPIOB0->CONHR = (GPIOB0->CONHR&0XFFFF00FF)  | 0x00005500; 
    }
    if (USART_IO_G==USART_TxPB0_7_RxPB0_6)
    { 
		GPIOB0->CONLR = (GPIOB0->CONLR&0X00FFFFFF)  | 0x44000000; 
    }
}
/*************************************************************/
//USART IO Init
//EntryParameter:
//ReturnValue:NONE
/*************************************************************/
void USART_CLKIO_Init(USART_CLKNUM_TypeDef USART_CLKIO_G)
{
	if (USART_CLKIO_G==CLK_PB0_8)
    {
		GPIOB0->CONHR = (GPIOB0->CONHR&0XFFFFFFF0)  | 0x00000004;       
    }
}
/*************************************************************/
//USART Software reset
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_Software_Reset(void)
{
	USART0->SRR =0x01;
}
/*************************************************************/
//USART close
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_CLK_Disable(void)
{
   USART0->CEDR |= 0; 							//Disable USART Clock 
}
/*************************************************************/
//USART CLK enable
//EntryParameter:INT_RegVal
//ReturnValue:NONE
/*************************************************************/
void USART_CLK_Enable(void)
{
   USART0->CEDR |= 1; 							//Disable USART Clock 
}
/*************************************************************/
//USART INT config
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_INT_Config( USART_INT_TypeDef int_type,FunctionalStatus newState)
{
	CSP_USART_SET_ICR(USART0, 0xFFFFFFFF);			// Clear all interrupt
	if (newState != DISABLE)
	{
		USART0->IMSCR |=int_type;					// Interrupt Enable
	}
	else
	{
		USART0->IMSCR &=~int_type;					// Interrupt Disable
	}
}
/*************************************************************/
//USART INT config from INT_RegVal
//EntryParameter:INT_RegVal
//ReturnValue:NONE
/*************************************************************/
void USART_INT_RegConfig(U32_T INT_RegVal)
{
	CSP_USART_SET_ICR(USART0, 0xFFFFFFFF);
	CSP_USART_SET_IMSCR(USART0, INT_RegVal);

}
/*************************************************************/
//USART mode config
//EntryParameter:INT_RegVal
//ReturnValue:NONE
/*************************************************************/
void USART_MODE_Config(USART_DSB_TypeDef DSB_DATA,USART_CLKS_TypeDef CLKS_DATA, USART_CHRL_TypeDef CHRL_DATA,
						USART_SYNC_TypeDef SYNC_DATA,USART_PAR_TypeDef PAR_DAT,USART_NBSTOP_TypeDef NBSTOP_DATA,USART_CHMODE_TypeDef CHMODE_DATA,
						USART_FIFO_CMD_TypeDef USART_FIFO_CMD , USART_FIFO_TRIG_TypeDef USART_FIFO_TRIG)
{
	USART0->MR = (USART0->MR&0X0FED000F)|CLKS_DATA|CHRL_DATA|DSB_DATA|SYNC_DATA|PAR_DAT|NBSTOP_DATA|CHMODE_DATA|USART_FIFO_CMD|USART_FIFO_TRIG ;
}
/*************************************************************/
//USART control regster config
//EntryParameter:CTRL_DATA, ENABLE/DISABLE
//ReturnValue:NONE
/*************************************************************/
void USART_CTRL_Config(USART_CTRL_TypeDef CTRL_DATA , FunctionalStatus newState)
{
	if (newState!=DISABLE)
	{
		USART0->CR  |= 1<<CTRL_DATA;
	}
	else
	{
		USART0->CR  |= 0<<CTRL_DATA;
	}
}
/*************************************************************/
//USART DSB CONFIG 
//EntryParameter:LSBtoMSB / MSBtoLSB
//ReturnValue:NONE
/*************************************************************/
void USART_DSB_Config(USART_DSB_TypeDef DSB_DATA)
{
	USART0->MR |= DSB_DATA;
}
/*************************************************************/
//USART SAMRTCARD Interface config
//EntryParameter:DISABLE/ENABLE
//ReturnValue:NONE
/*************************************************************/
void USART_SAMRTCARD_Config(FunctionalStatus newState)
{
	if (newState!=DISABLE)
	{
		USART0->MR |= 1<<16;
	}
	else
	{
		USART0->MR |= 0<<16;
	}
}
/*************************************************************/
//USART CLK OUT config
//EntryParameter:DISABLE/ENABLE
//ReturnValue:NONE
/*************************************************************/
void USART_CLKO_Config(FunctionalStatus newState)
{
	if (newState!=DISABLE)
	{
		USART0->MR |= (1<<18) | (0<<4) ;
	}
	else
	{
		USART0->MR |= 0<<18;
	}
}
/*************************************************************/
//USART baudrate calculate
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_Baudrate_Cal(unsigned int rate,unsigned int PCLK_Val,USART_CLKS_TypeDef CLKS_DATA,USART_SYNC_TypeDef SYNC_DATA)
{
	unsigned int PCLK_Data=0;
	unsigned int BRGR_Data=0;
	if (CLKS_DATA==PCLK_DIV8)
	{
		PCLK_Data=PCLK_Val/8;
	}
	else PCLK_Data=PCLK_Val;
	if (SYNC_DATA==SYNC)
	{
		BRGR_Data=PCLK_Data/rate;			//SYNC同步
	}
	else 
	{
		BRGR_Data=PCLK_Data/16/rate;				//ASYNC异步
	}
	USART0->BRGR = BRGR_Data<<4;
}
/*************************************************************/
//USART Receiver Time Out and Transmitter Timer Guard Initial
//EntryParameter:INT_RegVal
//ReturnValue:NONE
/*************************************************************/
void USART_TGTO_Config(U8_T timeOut_u8,U8_T timeGuard_u8)
{
    USART0->RTOR=timeOut_u8;					// Set Receiver Time Out
    USART0->TTGR=timeGuard_u8;					// Set Transmitter Timer Guard
}
/*************************************************************/
//USART TX byte
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void USART_TxByte(U16_T TXdata_u8)
{
	CSP_USART_SET_THR(USART0,TXdata_u8);		// Write the transmit buffer
	while ( (CSP_USART_GET_RISR(USART0) & USART_TXRDY) != USART_TXRDY ) {}	// Wait for the transmit done
	CSP_USART_SET_ICR(USART0, USART_TXRDY);		// Clear status bit for the next transmit
}
/*************************************************************/
//UART Transmit 
//EntryParameter:UART0,UART1,sourceAddress_u16,length_u16
//ReturnValue:NONE
/*************************************************************/
void USARTTransmit(U16_T *sourceAddress_u16,U16_T length_u16)
{
	unsigned int  DataJ;  //DataI,
	for(DataJ = 0;DataJ < length_u16 ;DataJ ++)
	{
		CSP_USART_SET_THR(USART0,*sourceAddress_u16++);
		/*do{
			DataI = CSP_USART_GET_RHR(USART0);
			DataI = DataI & USART_TXRDY;
		}while(DataI == USART_TXRDY);    //Loop  when tx is full
		*/
		while ( (CSP_USART_GET_RISR(USART0) & USART_TXRDY) != USART_TXRDY ) {}	// Wait for the transmit done
		CSP_USART_SET_ICR(USART0, USART_TXRDY);		// Clear status bit for the next transmit
	}
}
/*************************************************************/
//UART RX Byte 
//EntryParameter:UART0,UART1,Rxdata_u16
//ReturnValue:NONE
/*************************************************************/
U16_T USARTRxByte(CSP_UART_T *uart,U16_T *Rxdata_u16)
{
	unsigned int  DataI;

	DataI = CSP_USART_GET_RHR(USART0);
	DataI = DataI & USART_RXRDY;
	if(DataI != USART_RXRDY)    //Loop  when rx is not full
		return FALSE;
	else
	{
		*Rxdata_u16 = CSP_USART_GET_RHR(USART0);
	    return TRUE;
	}
}

/*************************************************************/
//USART Return RX byte by interrupt
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
U16_T USART_ReturnRxByte(void)
{
	USART_RxDataFlag = FALSE;
	while(USART_RxDataFlag != TRUE);
	return CSP_USART_GET_RHR(USART0);
}
/*************************************************************/
//USART Receive 
//EntryParameter:destAddress_u16,length_u16
//ReturnValue:NONE
/*************************************************************/
U16_T USARTReceive(U16_T *destAddress_u16,U16_T length_u16)
{
	unsigned int  DataI,DataJ,LoopTime;

	DataJ = 0;
	LoopTime = 0;
	do{
		DataI = CSP_USART_GET_RHR(USART0);
		DataI = DataI & USART_RXRDY;
		if(DataI == USART_RXRDY)    //Loop  when rx is full
		{
			*destAddress_u16++ = CSP_USART_GET_RHR(USART0);
			DataJ++;
			LoopTime = 0;
		}
		if(LoopTime ++ >= 0xfff0)
			return FALSE;
	}while(DataJ < length_u16);
	return TRUE;
}
/*************************************************************/
//LIN MODE CONFIG
//EntryParameter:destAddress_u16,length_u16
//ReturnValue:NONE
/*************************************************************/
//每个datax超时=LCP+SYNC_BRK-13
void LIN_MODE_CONFIG(LIN_VER_TypeDef LIN_VER_X , LIN_Checksum_Mode_TypeDef  LIN_Checksum_Mode_X , 
						volatile U8_T *byLCP , volatile U8_T bySyncBrk , volatile U16_T hwWakeupTime)
{
	USART0->LIR = (USART0->LIR&0X00000DFF)|(hwWakeupTime<<16)|LIN_Checksum_Mode_X;
	if((bySyncBrk < 13) || (bySyncBrk > 32))
		USART0->SBLR = 13;
	else
		USART0->SBLR	= bySyncBrk;
	USART0->LCP1=((*(byLCP+3))<<24)+((*(byLCP+2))<<16)+((*(byLCP+1))<<8)+(*byLCP);
	USART0->LCP2=((*(byLCP+7))<<24)+((*(byLCP+6))<<16)+((*(byLCP+5))<<8)+(*byLCP+4);
	USART0->MR = (USART0->MR&0xfff7fffe)|0X01|LIN_VER_X;			//使能LIN
}
/*************************************************************/
//LIN SET DATA ID
//EntryParameter:destAddress_u16,length_u16
//ReturnValue:NONE
/*************************************************************/
U8_T LIN_SET_DATA_ID(volatile U8_T byID , volatile U8_T *byLinData , volatile U8_T bySize)
{
	U8_T i;
	volatile uint32_t byVer = (USART0->MR&(0X01<<19));
	volatile uint8_t byDataNum;
	volatile uint32_t wData[2] = {0, 0};
	if(byVer==USART_LIN1_2)
	{
		if(bySize<4)
		{
			byDataNum=1;
			wData[0]=(uint32_t)(byLinData[0]|(byLinData[1]<<8));
			USART0->DFWR0=wData[0];
		}
		else if((bySize>=4)&&(bySize<8))
		{
			byDataNum=2;
			wData[0]=(uint32_t)(byLinData[0]|(byLinData[1]<<8)|(byLinData[1]<<16)|(byLinData[1]<<24));
			USART0->DFWR0=wData[0];
		}
		else
		{
			byDataNum=3;
			wData[0]=(uint32_t)(byLinData[0]|(byLinData[1]<<8)|(byLinData[1]<<16)|(byLinData[1]<<24));
			wData[1]=(uint32_t)(byLinData[4]|(byLinData[5]<<5)|(byLinData[6]<<6)|(byLinData[7]<<24));
			USART0->DFWR0=wData[0];
			USART0->DFWR1=wData[1];
		}
		USART0->LIR = (USART0->LIR & ~((0x03ul << 4)  | 0x0F)) | (((byDataNum) & 0x03) << 4) | (byID&0X0F);	//lin1.2
	}
	else if(byVer==USART_LIN2_0)
	{
		byDataNum=bySize-1;
		if(bySize < 5)
		{
			for(i = 0; i < bySize; i++)
			{
				wData[0] |= (*(byLinData + i) << (8 * i));
			}
			USART0->DFWR0=wData[0];
		}
		else
		{
			for(i = 0; i < (bySize - 4); i++)
			{
				wData[1] |= (*(byLinData + 4 + i) << (8 * i));
			}
				
			wData[0] = (uint32_t)(byLinData[0] | (byLinData[1] << 8) | (byLinData[2] << 16) | (byLinData[3] << 24));
			USART0->DFWR0=wData[0];
			USART0->DFWR1=wData[1];
		}
		USART0->LIR = (USART0->LIR & ~((0x07ul << 6)  | 0x3Ful)) | (((byDataNum) & 0x07) << 6) | (byID&0X3F);	//lin2.0
	}
	return byDataNum;
}
/*************************************************************/
//LIN DATA SEND
//EntryParameter:destAddress_u16,length_u16
//ReturnValue:NONE
/*************************************************************/
//LIN1.2 byID:0~0X0F
//LIN2.0 byID:0~0x3f
void LIN_DATA_SEND(volatile U8_T byID , volatile U8_T *byLinData , volatile U8_T bySize)
{
	volatile uint8_t byDataNum;
	byDataNum=LIN_SET_DATA_ID(byID,byLinData,bySize);
	if(byDataNum)
	{
		USART_CTRL_Config(LIN_STMESSAGE,ENABLE);
		asm ("nop");
	}
	//while ( (CSP_USART_GET_SR(USART) & LIN_ENDMESS_INT) != LIN_ENDMESS_INT );
	//while ( (CSP_USART_GET_SR(USART) & LIN_ENDHEADER_INT) != LIN_ENDHEADER_INT );	
}

/*************************************************************/
//LIN MASTER SEND ID RECdata Polling
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
U8_T LIN_MASTER_SENDid_RECdata_Polling (volatile U8_T byID , volatile U8_T *byRecLinData , volatile U8_T bySize)
{
	U32_T hTimeOutCnt=0;
	
	volatile uint32_t byVer = (USART0->MR&(0X01<<19));
	volatile uint8_t byDataNum;
	if(byVer==USART_LIN1_2)
	{
		if(bySize<4)
		{
			byDataNum=1;
		}
		else if((bySize>=4)&&(bySize<8))
		{
			byDataNum=2;
		}
		else
		{
			byDataNum=3;
		}
		USART0->LIR = (USART0->LIR & ~((0x03ul << 4)  | 0x0F)) | (((byDataNum) & 0x03) << 4) | (byID&0X0F);	//lin1.2
	}
	else if(byVer==USART_LIN2_0)
	{
		byDataNum=bySize-1;
		USART0->LIR = (USART0->LIR & ~((0x07ul << 6)  | 0x3Ful)) | (((byDataNum) & 0x07) << 6) | (byID&0X3F);	//lin2.0
	}
	if(byDataNum)
	{
		USART_CTRL_Config(LIN_STHEADER,ENABLE);
		asm ("nop");
	}
	do 
	{
		asm ("nop");
		hTimeOutCnt++;
		if(hTimeOutCnt>=0xfff)
		{
			return 0;
		}
	}
	while ( (CSP_USART_GET_SR(USART0) & LIN_ENDMESS_INT) != LIN_ENDMESS_INT );			//消息完成
	//while ( (CSP_USART_GET_SR(USART0) & LIN_ENDHEADER_INT) != LIN_ENDHEADER_INT );	
	if(byLin0EerorFlg!=0)
	{
		byRecLinData[0]=USART0->DFRR0&0XFF;
		byRecLinData[1]=(USART0->DFRR0>>8)&0XFF;
		byRecLinData[2]=(USART0->DFRR0>>16)&0XFF;
		byRecLinData[3]=(USART0->DFRR0>>24)&0XFF;
				
		byRecLinData[4]=USART0->DFRR1&0XFF;
		byRecLinData[5]=(USART0->DFRR1>>8)&0XFF;
		byRecLinData[6]=(USART0->DFRR1>>16)&0XFF;
		byRecLinData[7]=(USART0->DFRR1>>24)&0XFF;
		
		return 1;
	}
	return 0;
}

/*************************************************************/
//LIN MASTER SEND ID RECdata Int
//EntryParameter:NONE
//ReturnValue:NONE
/*************************************************************/
void LIN_MASTER_SENDid_RECdata_Int (volatile U8_T byID , volatile U8_T bySize)
{
	volatile uint32_t byVer = (USART0->MR&(0X01<<19));
	volatile uint8_t byDataNum;
		if(byVer==USART_LIN1_2)
		{
			if(bySize<4)
			{
				byDataNum=1;
			}
			else if((bySize>=4)&&(bySize<8))
			{
				byDataNum=2;
			}
			else
			{
				byDataNum=3;
			}
			USART0->LIR = (USART0->LIR & ~((0x03ul << 4)  | 0x0F)) | (((byDataNum) & 0x03) << 4) | (byID&0X0F);	//lin1.2
		}
		else if(byVer==USART_LIN2_0)
		{
			byDataNum=bySize-1;
			USART0->LIR = (USART0->LIR & ~((0x07ul << 6)  | 0x3Ful)) | (((byDataNum) & 0x07) << 6) | (byID&0X3F);	//lin2.0
		}
		if(byDataNum)
		{
			USART_CTRL_Config(LIN_STHEADER,ENABLE);
			asm ("nop");
		}
}

/******************* (C) COPYRIGHT 2023 APT Chip *****END OF FILE****/